@@ -5,6 +5,7 @@ struct KDTree{V <: AbstractVector, M <: MinkowskiMetric, T, TH} <: NNTree{V,M}
     metric::M
     split_vals::Vector{T}
     split_dims::Vector{UInt16}
+    hyper_rects::Vector{HyperRectangle{TH}}
     tree_data::TreeData
     reordered::Bool
 end
@@ -68,16 +69,17 @@ function KDTree(data::AbstractVector{V},
 
     # Create first bounding hyper rectangle that bounds all the input points
     hyper_rec = compute_bbox(data)
+    hyper_rects = Vector{typeof(hyper_rec)}(undef, tree_data.n_internal_nodes)
 
     # Call the recursive KDTree builder
-    build_KDTree(1, data, data_reordered, hyper_rec, split_vals, split_dims, indices, indices_reordered,
+    build_KDTree(1, data, data_reordered, hyper_rec, split_vals, split_dims, hyper_rects, indices, indices_reordered,
                  1:length(data), tree_data, reorder, parallel)
     if reorder
         data = data_reordered
         indices = indices_reordered
     end
 
-    KDTree(storedata ? data : similar(data, 0), hyper_rec, indices, metric, split_vals, split_dims, tree_data, reorder)
+    KDTree(storedata ? data : similar(data, 0), hyper_rec, indices, metric, split_vals, split_dims, hyper_rects, tree_data, reorder)
 end
 
 function KDTree(data::AbstractVecOrMat{T},
@@ -104,6 +106,7 @@ function build_KDTree(index::Int,
                       hyper_rec::HyperRectangle,
                       split_vals::Vector{T},
                       split_dims::Vector{UInt16},
+                      hyper_rects::Vector{<:HyperRectangle},
                       indices::Vector{Int},
                       indices_reordered::Vector{Int},
                       range,
@@ -128,31 +131,26 @@ function build_KDTree(index::Int,
 
     split_vals[index] = split_val
     split_dims[index] = split_dim
+    hyper_rects[index] = hyper_rec
 
-    # Call the left sub tree with an updated hyper rectangle
-    new_maxes = @inbounds setindex(hyper_rec.maxes, split_val, split_dim)
-    hyper_rec_left = HyperRectangle(hyper_rec.mins, new_maxes)
-
-    # Call the right sub tree with an updated hyper rectangle
-    new_mins = @inbounds setindex(hyper_rec.mins, split_val, split_dim)
-    hyper_rec_right = HyperRectangle(new_mins, hyper_rec.maxes)
+    hyper_rec_left, hyper_rec_right = split_hyperrectangle(hyper_rec, split_dim, split_val)
 
     parallel_threshold = 10 * tree_data.leafsize
 
     if parallel && Threads.nthreads() > 1 && n_p > parallel_threshold
         left_task = Threads.@spawn build_KDTree(getleft(index), data, data_reordered, hyper_rec_left, split_vals, split_dims,
-                                                indices, indices_reordered, first(range):mid_idx - 1, tree_data, reorder, parallel)
+                                                hyper_rects, indices, indices_reordered, first(range):mid_idx - 1, tree_data, reorder, parallel)
 
         build_KDTree(getright(index), data, data_reordered, hyper_rec_right, split_vals, split_dims,
-                    indices, indices_reordered, mid_idx:last(range), tree_data, reorder, parallel)
+                    hyper_rects, indices, indices_reordered, mid_idx:last(range), tree_data, reorder, parallel)
 
         fetch(left_task)
     else
         build_KDTree(getleft(index), data, data_reordered, hyper_rec_left, split_vals, split_dims,
-                    indices, indices_reordered, first(range):mid_idx - 1, tree_data, reorder, parallel)
+                    hyper_rects, indices, indices_reordered, first(range):mid_idx - 1, tree_data, reorder, parallel)
 
         build_KDTree(getright(index), data, data_reordered, hyper_rec_right, split_vals, split_dims,
-                    indices, indices_reordered, mid_idx:last(range), tree_data, reorder, parallel)
+                    hyper_rects, indices, indices_reordered, mid_idx:last(range), tree_data, reorder, parallel)
     end
 end
 
@@ -162,9 +160,10 @@ function _knn(tree::KDTree,
               best_idxs::Union{Integer, AbstractVector{<:Integer}},
               best_dists::Union{Number, AbstractVector},
               best_dists_final::Union{Nothing, AbstractVector},
-              skip::F) where {F}
+              skip::F,
+              self_idx::Int=0) where {F}
     init_min = get_min_distance_no_end(tree.metric, tree.hyper_rec, point)
-    best_idxs, best_dists = knn_kernel!(tree, 1, point, best_idxs, best_dists, init_min, tree.hyper_rec, skip, nothing)
+    best_idxs, best_dists = knn_kernel!(tree, 1, point, best_idxs, best_dists, init_min, tree.hyper_rec, skip, nothing, self_idx)
     best_dists isa Number && return best_idxs, eval_end(tree.metric, best_dists)
     @simd for i in eachindex(best_dists)
         @inbounds best_dists_final[i] = eval_end(tree.metric, best_dists[i])
@@ -180,10 +179,11 @@ function knn_kernel!(tree::KDTree{V},
                         min_dist,
                         hyper_rec::HyperRectangle,
                         skip::F,
-                        dedup::MaybeBitSet) where {V, F}
+                        dedup::MaybeBitSet,
+                        self_idx::Int=0) where {V, F}
     # At a leaf node. Go through all points in node and add those in range
     if isleaf(tree.tree_data.n_internal_nodes, index)
-        return add_points_knn!(best_dists, best_idxs, tree, index, point, false, skip, dedup)
+        return add_points_knn!(best_dists, best_idxs, tree, index, point, false, skip, dedup, self_idx)
     end
 
     split_dim = tree.split_dims[index]
@@ -191,20 +191,23 @@ function knn_kernel!(tree::KDTree{V},
     split_val = tree.split_vals[index]
     split_diff = p_dim - split_val
     M = tree.metric
+    left_region, right_region = split_hyperrectangle(hyper_rec, split_dim, split_val)
+    left_idx = getleft(index)
+    right_idx = getright(index)
     # Point is to the right of the split value
     if split_diff > 0
-        close = getright(index)
-        far = getleft(index)
-        hyper_rec_far = HyperRectangle(hyper_rec.mins, @inbounds setindex(hyper_rec.maxes, split_val, split_dim))
-        hyper_rec_close = HyperRectangle(@inbounds(setindex(hyper_rec.mins, split_val, split_dim)), hyper_rec.maxes)
+        close = right_idx
+        far = left_idx
+        hyper_rec_far = left_region
+        hyper_rec_close = right_region
     else
-        close = getleft(index)
-        far = getright(index)
-        hyper_rec_far = HyperRectangle(@inbounds(setindex(hyper_rec.mins, split_val, split_dim)), hyper_rec.maxes)
-        hyper_rec_close = HyperRectangle(hyper_rec.mins, @inbounds setindex(hyper_rec.maxes, split_val, split_dim))
+        close = left_idx
+        far = right_idx
+        hyper_rec_far = right_region
+        hyper_rec_close = left_region
     end
     # Always call closer sub tree
-    best_idxs, best_dists = knn_kernel!(tree, close, point, best_idxs, best_dists, min_dist, hyper_rec_close, skip, dedup)
+    best_idxs, best_dists = knn_kernel!(tree, close, point, best_idxs, best_dists, min_dist, hyper_rec_close, skip, dedup, self_idx)
 
     if M isa Chebyshev
         new_min = get_min_distance_no_end(M, hyper_rec_far, point)
@@ -214,7 +217,7 @@ function knn_kernel!(tree::KDTree{V},
 
     best_dist_1 = first(best_dists)
     if new_min < best_dist_1
-        best_idxs, best_dists = knn_kernel!(tree, far, point, best_idxs, best_dists, new_min, hyper_rec_far, skip, dedup)
+        best_idxs, best_dists = knn_kernel!(tree, far, point, best_idxs, best_dists, new_min, hyper_rec_far, skip, dedup, self_idx)
     end
     return best_idxs, best_dists
 end
@@ -268,16 +271,19 @@ function inrange_kernel!(
 
     count = 0
 
+    left_region, right_region = split_hyperrectangle(hyper_rec, split_dim, split_val)
+    left_idx = getleft(index)
+    right_idx = getright(index)
     if split_diff > 0 # Point is to the right of the split value
-        close = getright(index)
-        far = getleft(index)
-        hyper_rec_far = HyperRectangle(hyper_rec.mins, @inbounds setindex(hyper_rec.maxes, split_val, split_dim))
-        hyper_rec_close = HyperRectangle(@inbounds(setindex(hyper_rec.mins, split_val, split_dim)), hyper_rec.maxes)
+        close = right_idx
+        far = left_idx
+        hyper_rec_far = left_region
+        hyper_rec_close = right_region
     else # Point is to the left of the split value
-        close = getleft(index)
-        far = getright(index)
-        hyper_rec_far = HyperRectangle(@inbounds(setindex(hyper_rec.mins, split_val, split_dim)), hyper_rec.maxes)
-        hyper_rec_close = HyperRectangle(hyper_rec.mins, @inbounds setindex(hyper_rec.maxes, split_val, split_dim))
+        close = left_idx
+        far = right_idx
+        hyper_rec_far = right_region
+        hyper_rec_close = left_region
     end
     # Compute contributions for both close and far subtrees
     M = tree.metric
@@ -314,13 +320,6 @@ function inrange_kernel!(
 end
 # Self-query functions for finding all pairs within a tree
 
-@inline function _kdtree_child_rectangles(tree::KDTree, index::Int, rect::HyperRectangle)
-    split_dim = Int(tree.split_dims[index])
-    split_val = tree.split_vals[index]
-    left_rect = HyperRectangle(rect.mins, setindex(rect.maxes, split_val, split_dim))
-    right_rect = HyperRectangle(setindex(rect.mins, split_val, split_dim), rect.maxes)
-    return left_rect, right_rect
-end
 
 @inline function _order_pair(idx1::Int, rect1::HyperRectangle, idx2::Int, rect2::HyperRectangle)
     if idx1 <= idx2
@@ -354,17 +353,17 @@ function _inrange_kdtree_self!(results,
         if leaf_other
             _add_kdtree_self_leaf_pairs!(results, tree, idx, other_idx, r, skip)
         else
-            left_other, right_other = _kdtree_child_rectangles(tree, other_idx, other_rect)
+            left_other, right_other = split_hyperrectangle(other_rect, tree.split_dims[other_idx], tree.split_vals[other_idx])
             _inrange_kdtree_self!(results, tree, idx, rect, getleft(other_idx), left_other, r, skip)
             _inrange_kdtree_self!(results, tree, idx, rect, getright(other_idx), right_other, r, skip)
         end
     else
-        left_rect, right_rect = _kdtree_child_rectangles(tree, idx, rect)
+        left_rect, right_rect = split_hyperrectangle(rect, tree.split_dims[idx], tree.split_vals[idx])
         if leaf_other
             _inrange_kdtree_self!(results, tree, getleft(idx), left_rect, other_idx, other_rect, r, skip)
             _inrange_kdtree_self!(results, tree, getright(idx), right_rect, other_idx, other_rect, r, skip)
         else
-            left_other, right_other = _kdtree_child_rectangles(tree, other_idx, other_rect)
+            left_other, right_other = split_hyperrectangle(other_rect, tree.split_dims[other_idx], tree.split_vals[other_idx])
             if idx == other_idx
                 _inrange_kdtree_self!(results, tree, getleft(idx), left_rect, getleft(other_idx), left_other, r, skip)
                 _inrange_kdtree_self!(results, tree, getleft(idx), left_rect, getright(other_idx), right_other, r, skip)
